using System;
using System.Collections.Generic;
using System.Text;

using NBear.Common;
using NBear.IoC.Service;
using NBear.Data;
using SimpleGuestbook.Entities;
using SimpleGuestbook.ServiceInterfaces;

namespace SimpleGuestbook.ServiceImpls
{
    public class MessageService : IMessageService
    {
        #region IMessageService Members

        public SimpleGuestbook.Entities.Message[] GetMessages(int pageSize, int pageNo, bool includePrivate)
        {
            if (includePrivate)
            {
                return Gateway.Default.GetPageSelector<Message>(WhereClip.All, Message._.PostTime.Desc, pageSize).FindPage(pageNo);
            }
            else
            {
                return Gateway.Default.GetPageSelector<Message>(Message._.IsPrivate == false, Message._.PostTime.Desc, pageSize).FindPage(pageNo);
            }
        }

        public void AddMessage(SimpleGuestbook.Entities.Message msg)
        {
            Gateway.Default.Save<Message>(msg);
        }

        public void UpdateAdminFeedback(int msgID, string feedbackContent)
        {
            Message msg = Gateway.Default.Find<Message>(msgID);
            msg.AdminFeedback = feedbackContent;
            msg.AdminFeedbackTime = DateTime.Now;
            Gateway.Default.Save<Message>(msg);
        }

        public void DeleteMessage(int id)
        {
            Gateway.Default.Delete<Message>(id);
        }

        public int GetMessagePageCount(int pageSize, bool includePrivate)
        {
            if (includePrivate)
            {
                return Gateway.Default.GetPageSelector<Message>(WhereClip.All, OrderByClip.Default, pageSize).PageCount;
            }
            else
            {
                return Gateway.Default.GetPageSelector<Message>(Message._.IsPrivate == false, OrderByClip.Default, pageSize).PageCount;
            }
        }

        #endregion
    }

    public class AdminService : IAdminService
    {
        #region IAdminService Members

        public bool VerifyPassword(string password)
        {
            string hashedPassword = new CryptographyManager().ComputeHash(password);
            return Gateway.Default.Find<Admin>(Admin._.Password == hashedPassword) != null;
        }

        public void ResetPassword(string password)
        {
            Admin admin = new Admin();
            admin.Password = new CryptographyManager().ComputeHash(password);

            Gateway.Default.Delete<Admin>(WhereClip.All);
            Gateway.Default.Save<Admin>(admin);
        }

        #endregion
    }
}
